/*
 * Support string arrays as char ** input arguments.
 */
%typemap(in, numinputs=1, noblock=1) (const char **channel_names, size_t count_channels) {
	size_t count_channels;
	{
		if (!$input.is_cellstr())
			SWIG_exception(SWIG_TypeError, "channel_names must be a cell array of strings");
		Array<std::string> cellstr = $input.cellstr_value();

		count_channels = cellstr.nelem();
		$2 = count_channels;
		$1 = (char **) calloc(count_channels, sizeof(char *));

		for (octave_idx_type i = 0; i < cellstr.nelem(); i ++)
		{
			std::string str = cellstr(i);
			size_t len = str.size() + 1;
			$1[i] = (char *) calloc(len, sizeof(char));
			memcpy($1[i], str.c_str(), len);
		}
	}
}

%typemap(freearg, noblock=1) (const char **channel_names, size_t count_channels) {
	for (size_t i = 0; i < count_channels; i ++)
	{
		free($1[i]);
	}
	free($1);
}

/**
 * Support nds_channel and nds_buffer arrays double-pointer return values.
 */

%typemap(in, numinputs=0, noblock=1) size_t *count_channels {
	size_t count_channels;
	$1 = &count_channels;
}

%typemap(out, noblock=1) SWIGTYPE ** (octave_value_list outlist) {
	for (size_t i = 0; i < count_channels; i ++)
		outlist.append(SWIG_NewPointerObj($1[i], $*1_descriptor, 1));
	$result = static_cast<octave_value>(outlist);
}

%typemap(newfree, noblock=1) SWIGTYPE ** {
	free($1);
}

/* Use methods named 'toString' to supply string representations of objects. */
%rename toString __str;

%apply SWIGTYPE ** {nds_channel **, nds_buffer **};

/**
 * Make buffer.getChannel return a copy.
 */

%typemap(out, noblock=1) nds_channel * (nds_channel *ptr) {
	ptr = (nds_channel *) calloc(1, sizeof(nds_channel));
	memcpy(ptr, $1, sizeof(nds_channel));
	$result = SWIG_NewPointerObj(ptr, $1_descriptor, 1);
}

/**
 * Support getting numeric arrays from unsigned char[] using polymorphism.
 */

%typemap(out, noblock=1) unsigned char [] {
	switch (arg1->channel.data_type)
	{
		case NDS_DATA_TYPE_INT16:
		{
			Array<octave_int16> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(octave_int16));
			$result = static_cast<ArrayN<octave_int16> >(arr);
		} break;
		case NDS_DATA_TYPE_INT32:
		{
			Array<octave_int32> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(octave_int32));
			$result = static_cast<ArrayN<octave_int32> >(arr);
		} break;
		case NDS_DATA_TYPE_INT64:
		{
			Array<octave_int64> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(octave_int64));
			$result = static_cast<ArrayN<octave_int64> >(arr);
		} break;
		case NDS_DATA_TYPE_FLOAT32:
		{
			Array<float> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(float));
			$result = static_cast<ArrayN<float> >(arr);
		} break;
		case NDS_DATA_TYPE_FLOAT64:
		{
			Array<double> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(double));
			$result = static_cast<ArrayN<double> >(arr);
		} break;
		case NDS_DATA_TYPE_COMPLEX32:
		{
			Array<FloatComplex> arr(arg1->length);
			memcpy((void *) arr.data(), $1, arg1->length * sizeof(double));
			$result = static_cast<ArrayN<FloatComplex> >(arr);
		} break;
		default:
			SWIG_exception(SWIG_TypeError, "Unknown NDS data type");
			break;
	}
}
